@@ -1,8 +1,8 @@
 # Module: FeatureEngineering
 # Author: Adrian Antico <adrianantico@gmail.com>
 # License: MIT
-# Release: retrofit 0.0.8
-# Last modified : 2021-08-31
+# Release: retrofit 0.0.9
+# Last modified : 2021-09-02
 
 def FE0_AutoLags(data = None, ArgsList=None, LagColumnNames = None, DateColumnName = None, ByVariables = None, LagPeriods = 1, ImputeValue = -1, Sort = True, Processing='datatable', InputFrame='datatable', OutputFrame='datatable'):
     """
@@ -26,6 +26,7 @@ def FE0_AutoLags(data = None, ArgsList=None, LagColumnNames = None, DateColumnNa
     OutputFrame:    'datatable', 'polars', or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
     # QA: Test AutoLags
+    import timeit
     import datatable as dt
     from datatable import sort, f, by
     import retrofit
@@ -34,19 +35,28 @@ def FE0_AutoLags(data = None, ArgsList=None, LagColumnNames = None, DateColumnNa
     ## Group Example:
     data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoLags(data=data, LagPeriods=[1,3,5,7], LagColumnNames='Leads', DateColumnName='CalendarDateColumn', ByVariables=None, ImputeValue=-1, Sort=True, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## Group and Multiple Periods and LagColumnNames:
     data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoLags(data=data, LagPeriods=[1,3,5], LagColumnNames=['Leads','XREGS1'], DateColumnName='CalendarDateColumn', ByVariables=['MarketingSegments', 'MarketingSegments2', 'MarketingSegments3', 'Label'], ImputeValue=-1, Sort=True, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## No Group Example:
     data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoLags(data=data, LagPeriods=1, LagColumnNames='Leads', DateColumnName='CalendarDateColumn', ByVariables=None, ImputeValue=-1, Sort=True, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
     
     # QA: No Group Case: Step through function
@@ -129,9 +139,11 @@ def FE0_AutoLags(data = None, ArgsList=None, LagColumnNames = None, DateColumnNa
         SortCols = copy.copy(ByVariables)
         SortCols.append(DateColumnName)
         rev = [True for t in range(len(SortCols))]
-        data = data.sort(SortCols, reverse = rev)
+        data.sort(SortCols, reverse = rev, in_place = True)
       else:
-        data = data.sort(DateColumnName, reverse = True)
+        if not isinstance(data[DateColumnName].dtype(), pl.Date32):
+          data[DateColumnName] = data[DateColumnName].cast(pl.Date32)
+        data.sort(DateColumnName, reverse = True, in_place = True)
 
     # Ensure List
     if not LagColumnNames is None and not isinstance(LagColumnNames, list):
@@ -219,6 +231,7 @@ def FE0_AutoRollStats(data = None, ArgsList=None, RollColumnNames = None, DateCo
     OutputFrame:      'datatable' or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
     # QA AutoRollStats
+    import timeit
     import datatable as dt
     from datatable import sort, f, by
     import retrofit
@@ -226,17 +239,26 @@ def FE0_AutoRollStats(data = None, ArgsList=None, RollColumnNames = None, DateCo
 
     ## No Group Example
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoRollStats(data=data, RollColumnNames='Leads', DateColumnName='CalendarDateColumn', ByVariables=None, MovingAvg_Periods=[3,5,7], MovingSD_Periods=[3,5,7], MovingMin_Periods=[3,5,7], MovingMax_Periods=[3,5,7], ImputeValue=-1, Sort=True)
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## Group and Multiple Periods and RollColumnNames:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoRollStats(data=data, RollColumnNames=['Leads','XREGS1'], DateColumnName='CalendarDateColumn', ByVariables=['MarketingSegments', 'MarketingSegments2', 'MarketingSegments3', 'Label'], MovingAvg_Periods=[3,5,7], MovingSD_Periods=[3,5,7], MovingMin_Periods=[3,5,7], MovingMax_Periods=[3,5,7], ImputeValue=-1, Sort=True)
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## No Group Example:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoRollStats(data=data, RollColumnNames='Leads', DateColumnName='CalendarDateColumn', ByVariables=None, MovingAvg_Periods=[3,5,7], MovingSD_Periods=[3,5,7], MovingMin_Periods=[3,5,7], MovingMax_Periods=[3,5,7], ImputeValue=-1, Sort=True)
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
     
     # QA: No Group Case: Step through function
@@ -329,9 +351,11 @@ def FE0_AutoRollStats(data = None, ArgsList=None, RollColumnNames = None, DateCo
         SortCols = copy.copy(ByVariables)
         SortCols.append(DateColumnName)
         rev = [True for t in range(len(SortCols))]
-        data = data[:, :, sort(SortCols, reverse=rev)]
+        data.sort(SortCols, reverse = rev, in_place = True)
       else:
-        data = (data.sort(DateColumnName, reverse = True))
+        if not isinstance(data[DateColumnName].dtype(), pl.Date32):
+          data[DateColumnName] = data[DateColumnName].cast(pl.Date32)
+        data.sort(DateColumnName, reverse = True, in_place = True)
 
     # Prepare column and value references
     if not RollColumnNames is None and not isinstance(RollColumnNames, list):
@@ -442,6 +466,7 @@ def FE0_AutoDiff(data = None, ArgsList = None, DateColumnName = None, ByVariable
     OutputFrame:          'datatable' or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
     # QA AutoDiff
+    import timeit
     import datatable as dt
     from datatable import sort, f, by
     import retrofit
@@ -449,17 +474,26 @@ def FE0_AutoDiff(data = None, ArgsList = None, DateColumnName = None, ByVariable
 
     ## Group Example:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoDiff(data=data, ArgsList=None, DateColumnName = 'CalendarDateColumn', ByVariables = ['MarketingSegments', 'MarketingSegments2', 'MarketingSegments3', 'Label'], DiffNumericVariables = 'Leads', DiffDateVariables = 'CalendarDateColumn', DiffGroupVariables = None, NLag1 = 0, NLag2 = 1, Sort=True, InputFrame = 'datatable', OutputFrame = 'datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## Group and Multiple Periods and RollColumnNames:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoDiff(data=data, ArgsList=None, DateColumnName = 'CalendarDateColumn', ByVariables = ['MarketingSegments', 'MarketingSegments2', 'MarketingSegments3', 'Label'], DiffNumericVariables = 'Leads', DiffDateVariables = 'CalendarDateColumn', DiffGroupVariables = None, NLag1 = 0, NLag2 = 1, Sort=True, InputFrame = 'datatable', OutputFrame = 'datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     ## No Group Example:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE0_AutoDiff(data=data, ArgsList=None, DateColumnName = 'CalendarDateColumn', ByVariables = None, DiffNumericVariables = 'Leads', DiffDateVariables = 'CalendarDateColumn', DiffGroupVariables = None, NLag1 = 0, NLag2 = 1, Sort=True, InputFrame = 'datatable', OutputFrame = 'datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
     
     # QA: No Group Case: Step through function
@@ -562,9 +596,11 @@ def FE0_AutoDiff(data = None, ArgsList = None, DateColumnName = None, ByVariable
         SortCols = copy.copy(ByVariables)
         SortCols.append(DateColumnName)
         rev = [True for t in range(len(SortCols))]
-        data = data[:, :, sort(SortCols, reverse=rev)]
+        data.sort(SortCols, reverse = rev, in_place = True)
       else:
-        data = (data.sort(DateColumnName, reverse = True))
+        if not isinstance(data[DateColumnName].dtype(), pl.Date32):
+          data[DateColumnName] = data[DateColumnName].cast(pl.Date32)
+        data.sort(DateColumnName, reverse = True, in_place = True)
 
     # DiffNumericVariables
     if not DiffNumericVariables is None:
@@ -713,6 +749,7 @@ def FE1_AutoCalendarVariables(data = None, ArgsList = None, DateColumnNames = No
     OutputFrame:          'datatable' or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
     # QA AutoCalendarVariables
+    import timeit
     import datatable as dt
     from datatable import sort, f, by, ifelse
     import retrofit
@@ -720,7 +757,10 @@ def FE1_AutoCalendarVariables(data = None, ArgsList = None, DateColumnNames = No
 
     ## Example:
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     data = fe.FE1_AutoCalendarVariables(data=data, ArgsList=None, DateColumnNames = 'CalendarDateColumn', CalendarVariables = ['wday','mday','wom','month','quarter','year'], Processing = 'datatable', InputFrame = 'datatable', OutputFrame = 'datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     print(data.names)
 
     # QA: No Group Case: Step through function
@@ -846,6 +886,7 @@ def FE1_DummyVariables(data=None, ArgsList=None, CategoricalColumnNames=None, Pr
     OutputFrame:            'datatable', 'polars', or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
     # Example: datatable
+    import timeit
     import datatable as dt
     import retrofit
     from retrofit import FeatureEngineering as fe
@@ -855,11 +896,15 @@ def FE1_DummyVariables(data=None, ArgsList=None, CategoricalColumnNames=None, Pr
     ArgsList = Output['ArgsList']
     
     # Example: polars
+    import timeit
     import retrofit
     from retrofit import FeatureEngineering as fe
     import polars as pl
     data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
     Output = fe.FE1_DummyVariables(data=data, ArgsList=None, CategoricalColumnNames=['MarketingSegments','MarketingSegments2'], Processing='polars', InputFrame='polars', OutputFrame='polars')
+    t_end = timeit.default_timer()
+    t_end - t_start
     data = Output['data']
     ArgsList = Output['ArgsList']
     
@@ -927,7 +972,7 @@ def FE1_DummyVariables(data=None, ArgsList=None, CategoricalColumnNames=None, Pr
     elif Processing.lower() == 'polars':
       return dict(data = data, ArgsList = ArgsList)
 
-def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, PartitionType='random', Ratios=None, ByVariables=None, Processing='datatable', InputFrame='datatable', OutputFrame='datatable'):
+def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, PartitionType='random', Ratios=None, ByVariables=None, Sort=False, Processing='datatable', InputFrame='datatable', OutputFrame='datatable'):
     
     """
     # Goal:
@@ -942,12 +987,14 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
     DateColumnName: Scalar. Primary date column used for sorting
     PartitionType:  Scalar. Columns to partition over
     Ratios:         List. Use ths for PartitionType 'random'. List of decimal values for determining how many data goes into each data frame.
+    Sort:           Sort data before creating time based partitions
     ByVariables:    None or List. Stratify the data paritioning using ByVariables
     Processing:     'datatable' or 'polars'. Choose the package you want to do your processing
     InputFrame:     'datatable', 'polars', or 'pandas' If you input Frame is 'pandas', it will be converted to a datatable Frame for generating the new columns
     OutputFrame:    'datatable', 'polars', or 'pandas' If you want the output Frame to be pandas change value to 'pandas'
     
-    # Example
+    # datatable Example
+    import timeit
     import datatable as dt
     import retrofit
     from retrofit import FeatureEngineering as fe
@@ -955,7 +1002,10 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
     
     # random
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
-    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='random', Ratios=[0.70,0.20,0.10], ByVariables=None, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_start = timeit.default_timer()
+    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='random', Ratios=[0.70,0.20,0.10], Sort=False, ByVariables=None, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     TrainData = DataSets['TrainData']
     ValidationData = DataSets['ValidationData']
     TestData = DataSets['TestData']
@@ -963,13 +1013,46 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
     
     # time
     data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
-    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='time', Ratios=[0.70,0.20,0.10], ByVariables=None, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_start = timeit.default_timer()
+    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='time', Ratios=[0.70,0.20,0.10], Sort=True, ByVariables=None, Processing='datatable', InputFrame='datatable', OutputFrame='datatable')
+    t_end = timeit.default_timer()
+    t_end - t_start
     TrainData = DataSets['TrainData']
     ValidationData = DataSets['ValidationData']
     TestData = DataSets['TestData']
     ArgsList = DataSets['ArgsList']
     
-    # QA
+    # polars Example
+    import timeit
+    import polars as pl
+    import retrofit
+    from retrofit import FeatureEngineering as fe
+    from retrofit import utils as u
+    
+    # random
+    data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/RemixAutoML/tests/QA_DataSets/ThreeGroup-FC-Walmart-XREG3.csv")
+    data = pl.read_csv('https://www.dropbox.com/s/2str3ek4f4cheqi/walmart_train.csv?dl=1')
+    data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
+    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='random', Ratios=[0.70,0.20,0.10], Sort=False, ByVariables=None, Processing='polars', InputFrame='polars', OutputFrame='polars')
+    t_end = timeit.default_timer()
+    t_end - t_start
+    TrainData = DataSets['TrainData']
+    ValidationData = DataSets['ValidationData']
+    TestData = DataSets['TestData']
+    ArgsList = DataSets['ArgsList']
+    
+    # time
+    data = dt.fread("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    data = pl.read_csv("C:/Users/Bizon/Documents/GitHub/BenchmarkData.csv")
+    t_start = timeit.default_timer()
+    DataSets = fe.FE2_AutoDataParition(data=data, ArgsList=None, DateColumnName='CalendarDateColumn', PartitionType='time', Ratios=[0.70,0.20,0.10], Sort=True, ByVariables=None, Processing='polars', InputFrame='polars', OutputFrame='polars')
+    t_end = timeit.default_timer()
+    t_end - t_start
+    TrainData = DataSets['TrainData']
+    ValidationData = DataSets['ValidationData']
+    TestData = DataSets['TestData']
+    ArgsList = DataSets['ArgsList']
     
     # random
     ArgsList=None
@@ -981,6 +1064,10 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
     InputFrame='datatable'
     OutputFrame='datatable'
     
+    Processing='polars'
+    InputFrame='polars'
+    OutputFrame='polars'
+    
     # time
     ArgsList=None
     DateColumnName='CalendarDateColumn'
@@ -1017,7 +1104,6 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
     # Import polars methods
     if Processing.lower() == 'polars' or OutputFrame.lower() == 'polars' or InputFrame.lower() == 'polars':
       import polars as pl
-      from polars import col
       from polars.lazy import col
 
     # Convert to datatable
@@ -1028,53 +1114,121 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
 
     # Accumulate Ratios
     Ratios = u.cumsum(Ratios)
-
-    # Random partitioning
-    if PartitionType.lower() == 'random':
-
-      # Add random number column
-      data = data[:, f[:].extend({"ID": np.random.uniform(0,1, size = data.shape[0])})]
-
-      # TrainData
-      TrainData = data[f.ID <= Ratios[0], ...]
-      del TrainData['ID']
-
-      # ValidationData
-      ValidationData = data[(f.ID <= Ratios[1]) & (f.ID > Ratios[0]), ...]
-      del ValidationData['ID']
-
-      # TestData
-      if len(Ratios) == 3:
-        TestData = data[f.ID > Ratios[1], ...]
-        del TestData['ID']
-      else:
-        TestData = None
-
-    # Time base partitioning
-    if PartitionType.lower() == 'time':
+    
+    # datatable
+    if Processing.lower() == 'datatable':
       
-      # Sort data
-      if Sort == True and Processing.lower() == 'datatable':
-        data = data[:, :, sort(f[DateColumnName], reverse = False)]
-      elif Sort == True and Processing.lower() == 'polars':
-        data = (data.sort(f[DateColumnName], reverse = False))
+      # Random partitioning
+      if PartitionType.lower() == 'random':
+        
+        # Add random number column
+        data = data[:, f[:].extend({"ID": np.random.uniform(0,1, size = data.shape[0])})]
+        
+        # Sort data
+        data = data[:, :, sort(f['ID'])]
+        
+        # TrainData
+        TrainData = data[f['ID'] <= Ratios[0], ...]
+        del TrainData['ID']
+        
+        # ValidationData
+        ValidationData = data[(f['ID'] <= Ratios[1]) & (f['ID'] > Ratios[0]), ...]
+        del ValidationData['ID']
+        
+        # TestData
+        if len(Ratios) == 3:
+          TestData = data[f['ID'] > Ratios[1], ...]
+          del TestData['ID']
+        else:
+          TestData = None
 
-      # Grab row number boundaries
-      TrainRowsMax = data.nrows * Ratios[0]
-      ValidRowsMax = data.nrows * Ratios[1]
-      
-      # TrainData
-      TrainData = data[range(int(TrainRowsMax)), ...]
-      
-      # ValidationData
-      ValidationData = data[range(int(TrainRowsMax+1), int(ValidRowsMax)), ...]
-      
-      # TestData
-      if len(Ratios) == 3:
-        TestData = data[range(int(ValidRowsMax), data.nrows), ...]
-      else:
-        TestData = None
+      # Time base partitioning
+      if PartitionType.lower() == 'time':
+        
+        # Number of rows
+        NumRows = data.nrows
+        
+        # Sort data
+        if Sort == True:
+          data = data[:, :, sort(f[DateColumnName], reverse = False)]
+        
+        # Grab row number boundaries
+        TrainRowsMax = NumRows * Ratios[0]
+        ValidRowsMax = NumRows * Ratios[1]
+        
+        # TrainData
+        TrainData = data[range(int(TrainRowsMax)), ...]
+        
+        # ValidationData
+        ValidationData = data[range(int(TrainRowsMax+1), int(ValidRowsMax)), ...]
+        
+        # TestData
+        if len(Ratios) == 3:
+          TestData = data[range(int(ValidRowsMax), NumRows), ...]
+        else:
+          TestData = None
 
+    # polars
+    if Processing.lower() == 'polars':
+      
+      # Random partitioning
+      if PartitionType.lower() == 'random':
+        
+        # Add random number column
+        data['ID'] = np.random.uniform(0,1, size = data.shape[0])
+        
+        # Sort data
+        data = data.sort('ID')
+        
+        # TrainData
+        TrainData = data[data['ID'] <= Ratios[0]]
+        TrainData.drop_in_place('ID')
+        
+        # ValidationData
+        ValidationData = data[(data['ID'] <= Ratios[1]) & (data['ID'] > Ratios[0])]
+        ValidationData.drop_in_place('ID')
+        
+        # TestData
+        if len(Ratios) == 3:
+          TestData = data[data['ID'] > Ratios[1]]
+          TestData.drop_in_place('ID')
+        else:
+          TestData = None
+
+      # Time base partitioning
+      if PartitionType.lower() == "time":
+        
+        # Number of rows
+        NumRows = data.shape[0]
+        
+        # Sort data
+        if Sort == True:
+          data.sort(DateColumnName, reverse = False, in_place = True)
+
+        # Add an index to filter by
+        data = (data.with_column(
+          pl.arange(0,data.shape[0]).alias("index")
+        ))
+
+        # Grab row number boundaries
+        TrainRowsMax = NumRows * Ratios[0]
+        ValidRowsMax = NumRows * Ratios[1]
+        
+        # TrainData
+        TrainData = data[data['index'] <= int(TrainRowsMax)]
+        TrainData.drop_in_place('index')
+        
+        # ValidationData
+        ValidationData = data[(data['index'] > int(TrainRowsMax + 1)) & (data['index'] <= int(ValidRowsMax))]
+        ValidationData.drop_in_place('index')
+        
+        # TestData
+        if len(Ratios) == 3:
+          TestData = data[data['index'] > int(ValidRowsMax + 1)]
+          TestData.drop_in_place('index')
+        else:
+          TestData = None
+    
     # Convert Frame
     if OutputFrame.lower() == 'pandas' and (Processing.lower() == 'datatable' or Processing.lower() == 'polars'):
       TrainData = TrainData.to_pandas()
@@ -1089,6 +1243,6 @@ def FE2_AutoDataParition(data=None, ArgsList=None, DateColumnName=None, Partitio
       if len(Ratios) == 3:
         TestData = TestData.to_pandas()
         TestData = dt.Frame(TestData)
-
+    
     # Return data
     return dict(TrainData = TrainData, ValidationData = ValidationData, TestData = TestData, ArgsList = ArgsList)
